@@ -270,7 +270,11 @@ g_io_modules_scan_all_in_directory (const char *dirname)
   const gchar *name;
   char *filename;
   GDir *dir;
+#ifdef G_OS_WIN32
+  struct _g_stat_struct statbuf;
+#else
   struct stat statbuf;
+#endif
   char *data;
   time_t cache_mtime;
   GHashTable *cache;
@@ -321,14 +325,14 @@ g_io_modules_scan_all_in_directory (const char *dirname)
 	    continue; /* Invalid line, ignore */
 
 	  *colon = 0; /* terminate filename */
-	  file = strdup (line);
+	  file = g_strdup (line);
 	  colon++; /* after colon */
 
 	  while (g_ascii_isspace (*colon))
 	    colon++;
 
 	  extension_points = g_strsplit (colon, ",", -1);
-	  g_hash_table_insert (cache, g_strdup (file), extension_points);
+	  g_hash_table_insert (cache, file, extension_points);
 	}
       g_strfreev (lines);
     }
@@ -382,6 +386,8 @@ g_io_modules_scan_all_in_directory (const char *dirname)
   g_dir_close (dir);
 
   g_hash_table_destroy (cache);
+
+  g_free (filename);
 }
 
 
